package edu.lawrence.auction.repositories;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Profile;
import edu.lawrence.auction.entities.Purchase;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.Offer;
import edu.lawrence.auction.interfaces.dtos.Sale;
import edu.lawrence.auction.interfaces.dtos.Bill;
import edu.lawrence.auction.interfaces.dtos.ProfileDTO;
import edu.lawrence.auction.interfaces.dtos.ShippingDTO;
import edu.lawrence.auction.interfaces.dtos.UserDTO;
import edu.lawrence.auction.services.PasswordService;
import jakarta.persistence.EntityManager;
import jakarta.transaction.Transactional;

@Repository
public class UserDAO {

	@Autowired
	EntityManager em;
	
	@Autowired 
    PasswordService passwordService;
	
	@Transactional
	public String save(UserDTO user) {
		List<User> existing = em.createQuery("select u from User u where name=:name",User.class)
				.setParameter("name", user.getName()).getResultList();
		if(existing.size() > 0)
			return "Duplicate";
		
		User newUser = new User();
		newUser.setName(user.getName());
		String hash = passwordService.hashPassword(user.getPassword());
	    newUser.setPassword(hash);
		em.persist(newUser);
		return newUser.getUserid().toString();
	}
	
	@Transactional
	public User findByNameAndPassword(String name,String password) {
		List<User> existing = em.createQuery("select u from User u where name=:name",User.class)
				.setParameter("name", name).getResultList();
		if(existing.size() != 1)
			return null;
		User u = existing.get(0);
		if(passwordService.verifyHash(password, u.getPassword())) {
        	u.setPassword("Undisclosed");
        } else {
        	u = null;
        }
        return u;	
	}
	
	@Transactional
	public String saveProfile(UUID userid,ProfileDTO profile) {
		User user = em.find(User.class, userid);
		if(user == null)
			return "Bad Id";
		
		if(user.getProfile() != null)
			return "Duplicate";
		
		Shipping newShipping = new Shipping(profile.getShipping());
		newShipping.setUser(user);
		em.persist(newShipping);
		
		Profile newProfile = new Profile(profile);
		newProfile.setUser(user);
		newProfile.setShipping(newShipping);
		em.persist(newProfile);
		
		return "Created";
	}
	
	@Transactional
	public Profile findProfile(UUID userid) {
		User user = em.find(User.class, userid);
		if(user == null)
			return null;
		return user.getProfile();
	}
	
	@Transactional
	public String saveShipping(UUID userid,ShippingDTO shipping) {
		User user = em.find(User.class, userid);
		if(user == null)
			return "Bad Id";
		Shipping newShipping = new Shipping(shipping);
		newShipping.setUser(user);
		em.persist(newShipping);
		return "Created";
	}
	
	@Transactional
	public List<Shipping> findShipping(UUID userid) {
		User user = em.find(User.class, userid);
		if(user == null)
			return new ArrayList<Shipping>();
		return user.getShipping();
	}
	
	@Transactional
	public List<Auction> findAuctions(UUID userid) {
		User user = em.find(User.class, userid);
		if(user == null)
			return new ArrayList<Auction>();
		return user.getAuctions();
	}
	
	@Transactional
	public List<Offer> findOffers(UUID userid) {
		List<Purchase> purchases = em.createQuery("select p from Purchase p where p.status='Won_bid' and p.bid.bidder.userid=:user",Purchase.class)
				.setParameter("user",userid).getResultList();
		
		List<Offer> offers = new ArrayList<Offer>();
		for(Purchase p : purchases) {
			offers.add(new Offer(p));
		}
		return offers;
	}
	
	@Transactional
	public List<Sale> findSales(UUID userid) {
		List<Purchase> purchases = em.createQuery("select p from Purchase p where p.status='Confirmed' and p.bid.auction.seller.userid=:user",Purchase.class)
				.setParameter("user",userid).getResultList();
		
		List<Sale> sales = new ArrayList<Sale>();
		for(Purchase p : purchases) {
			sales.add(new Sale(p));
		}
		return sales;
	}
	
	@Transactional
	public List<Bill> findBills(UUID userid) {
		List<Purchase> purchases = em.createQuery("select p from Purchase p where p.status='Charged' and p.bid.bidder.userid=:user",Purchase.class)
				.setParameter("user",userid).getResultList();
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
	
	@Transactional
	public List<Bill> findSoldBills(UUID userid) {
		List<Purchase> purchases = em.createQuery("select p from Purchase p where p.status='Paid' and p.bid.auction.seller.userid=:user",Purchase.class)
				.setParameter("user",userid).getResultList();
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
	
	@Transactional
	public List<Bill> findShippedBills(UUID userid) {
		List<Purchase> purchases = em.createQuery("select p from Purchase p where p.status='Charged' and p.bid.bidder.userid=:user",Purchase.class)
				.setParameter("user",userid).getResultList();
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
}
